<?php
	// If this file is called directly, abort.
	if ( ! defined( 'WPINC' ) )
	{
		die;
	}

	/**
	 * The admin-specific functionality of the plugin.
	 *
	 * @link       https://www.codeboxr.com
	 * @since      1.0.0
	 *
	 * @package    CBXLatestTweets
	 * @subpackage CBXLatestTweets/admin
	 */

	/**
	 * The admin-specific functionality of the plugin.
	 *
	 * Defines the plugin name, version, and two examples hooks for how to
	 * enqueue the admin-specific stylesheet and JavaScript.
	 *
	 * @package    CBXLatestTweets
	 * @subpackage CBXLatestTweets/admin
	 * @author     Codeboxr <info@codeboxr.com>
	 */
	class CBXLatestTweets_Admin {

		/**
		 * The ID of this plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 * @var      string $plugin_name The ID of this plugin.
		 */
		private $plugin_name;

		/**
		 * The version of this plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 * @var      string $version The current version of this plugin.
		 */
		private $version;

		private $settings_api;

		/**
		 * Initialize the class and set its properties.
		 *
		 * @since    1.0.0
		 *
		 * @param      string $plugin_name The name of this plugin.
		 * @param      string $version     The version of this plugin.
		 */
		public function __construct( $plugin_name, $version ) {

			$this->plugin_name = $plugin_name;
			$this->version     = $version;

			//get instance of setting api
			$this->settings_api = new CBXLatestTweets_Settings_API( $this->plugin_name, $this->version );

		}

		/**
		 * Initialize setting
		 */
		public function setting_init() {
			//set the settings
			$this->settings_api->set_sections( $this->get_settings_sections() );
			$this->settings_api->set_fields( $this->get_settings_fields() );
			//initialize settings
			$this->settings_api->admin_init();
		}


		/**
		 * Global Setting Sections and titles
		 *
		 * @return type
		 */
		public function get_settings_sections() {
			$settings_sections = array(

				array(
					'id'    => 'cbxlatesttweets_api_config',
					'title' => esc_html__( 'Twitter API Config', 'cbxlatesttweets' )
				),

				array(
					'id'    => 'cbxlatesttweets_config',
					'title' => esc_html__( 'Default Tweets Config', 'cbxlatesttweets' )
				),
				array(
					'id'    => 'cbxlatesttweets_tools',
					'title' => esc_html__( 'Tools', 'cbxlatesttweets' )
				)

			);

			return apply_filters( 'cbxlatesttweets_setting_sections', $settings_sections );
		}

		/**
		 * Global Setting Fields
		 *
		 * @return array
		 */
		public function get_settings_fields() {

			global $wpdb;

			$reset_data_link = add_query_arg( 'cbxlatesttweets_fullreset', 1, admin_url( 'options-general.php?page=cbxlatesttweetssettings' ) );
			$reset_cache_link = add_query_arg( 'cbxlatesttweets_cachereset', 1, admin_url( 'options-general.php?page=cbxlatesttweetssettings' ) );

			$table_data_html = $table_cache_html = '';

			$table_data_html .= '<p id="cbxlatesttweets_tool_fullreset"><strong>' . esc_html__( 'Following option values created by this plugin(including addon)', 'cbxlatesttweets' ) . '</strong></p>';


			$option_values = CBXLatestTweetsHelper::getAllOptionNames();
			$table_counter = 1;
			foreach ( $option_values as $key => $value )
			{
				$table_data_html .= '<p>' . str_pad( $table_counter, 2, '0', STR_PAD_LEFT ) . '. ' . $value['option_name'] . ' - ' . $value['option_id'] . ' - (<code style="overflow-wrap: break-word; word-break: break-all;">' . $value['option_value'] . '</code>)</p>';

				$table_counter ++;
			}

			$table_cache_html .= '<p id="cbxlatesttweets_tool_cachereset"><strong>' . esc_html__( 'Following cache values created by this plugin(including addon)', 'cbxlatesttweets' ) . '</strong></p>';


			$transients_values = CBXLatestTweetsHelper::getAllTransients();

			$table_counter = 1;
			foreach ( $transients_values as $value )
			{
				$table_cache_html .= '<p>' . str_pad( $table_counter, 2, '0', STR_PAD_LEFT ) . '. ' . $value['option_id'] . ' - <code>' . $value['option_name'].'</code></p>';

				$table_counter ++;
			}

			$settings_builtin_fields =
				array(
					'cbxlatesttweets_api_config' => array(
						'api_help'        => array(
							'name'           => 'api_help',
							'label'          => esc_html__( 'Api Help', 'cbxlatesttweets' ),
							'desc'           => __('Please create app in <a href="https://apps.twitter.com/" target="_blank">Twitter Apps</a> and check <strong>Keys and Access Tokens</strong> tab for necessary api keys', 'cbxlatesttweets'),
							'type'           => 'heading',
							'default'        => ''
						),
						'consumer_key'        => array(
							'name'           => 'consumer_key',
							'label'          => esc_html__( 'Consumer Key', 'cbxlatesttweets' ),
							'desc'           => esc_html__( 'Please put Consumer Key here, required field.', 'cbxlatesttweets' ),
							'type'           => 'text',
							'default'        => ''
						),
						'consumer_secret'     => array(
							'name'           => 'consumer_secret',
							'label'          => esc_html__( 'Consumer Secret', 'cbxlatesttweets' ),
							'desc'           => esc_html__( 'Please put Consumer Secret here, required field.', 'cbxlatesttweets' ),
							'type'           => 'text',
							'default'        => ''
						),
						'access_token'        => array(
							'name'           => 'access_token',
							'label'          => esc_html__( ' Access Token', 'cbxlatesttweets' ),
							'desc'           => esc_html__( 'Please put  Access Token here, required field.', 'cbxlatesttweets' ),
							'type'           => 'text',
							'default'        => ''
						),
						'access_token_secret' => array(
							'name'           => 'access_token_secret',
							'label'          => esc_html__( ' Access Token Secret', 'cbxlatesttweets' ),
							'desc'           => esc_html__( 'Please put  Access Token Secret here, required field.', 'cbxlatesttweets' ),
							'type'           => 'text',
							'default'        => ''
						),
					),
					'cbxlatesttweets_config'     => array(
						'include_rts' => array(
							'name'     => 'include_rts',
							'label'    => esc_html__( 'Include Retweets', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'Will retweets be included when query tweets?', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'default'  => 0,
							'options'  => array(
								1 => esc_html__( 'Yes', 'cbxlatesttweets' ),
								0 => esc_html__( 'No', 'cbxlatesttweets' ),
							)

						),
						'exclude_replies' => array(
							'name'     => 'exclude_replies',
							'label'    => esc_html__( 'Exclude Replies', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'Will replies be included when query tweets?', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'default'  => 1,
							'options'  => array(
								1 => esc_html__( 'Yes', 'cbxlatesttweets' ),
								0 => esc_html__( 'No', 'cbxlatesttweets' ),
							)

						),
						'time_format'      => array(
							'name'     => 'time_format',
							'label'    => esc_html__( 'Default time format', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'What will be the time format to show tweet?', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'options'  => array(
								1 => esc_html__( 'Relative (Example: 5 mins ago)', 'cbxlatesttweets' ),
								0 => esc_html__( 'Regular (Example: dd-mm-yyyy)', 'cbxlatesttweets' ),
							),
							'default'  => 1
						),
						'date_time_format'      => array(
							'name'     => 'date_time_format',
							'label'    => esc_html__( 'Date time display format', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'If time format is regular then date time display format.', 'cbxlatesttweets' ). ' <a  target="_blank" href="https://codex.wordpress.org/Formatting_Date_and_Time">'.esc_html__('Documentation on date and time formatting', 'cbxlatesttweets').'</a>',
							'type'     => 'text',
							//'default'  => get_option('date_format'). ' '.get_option('time_format')
							'default'  => CBXLatestTweetsHelper::getGlobalDateTimeFormat()
						),
						'linkify_link'     => array(
							'name'     => 'linkify_link',
							'label'    => esc_html__( 'Linkify Link', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'Linkify any link found in tweet text', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'options'  => array(
								1 => esc_html__( 'Yes', 'cbxlatesttweets' ),
								0 => esc_html__( 'No', 'cbxlatesttweets' ),
							),
							'default'  => 1
						),
						'linkify_mention'  => array(
							'name'     => 'linkify_mention',
							'label'    => esc_html__( 'Linkify mention', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'Linkify mention of the tweet.', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'options'  => array(
								1 => esc_html__( 'Yes', 'cbxlatesttweets' ),
								0 => esc_html__( 'No', 'cbxlatesttweets' ),
							),
							'default'  => 1
						),
						'linkify_hashtag'  => array(
							'name'     => 'linkify_hashtag',
							'label'    => esc_html__( 'Linkify hashtag', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'Linkify hashtag of the tweet.', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'options'  => array(
								1 => esc_html__( 'Yes', 'cbxlatesttweets' ),
								0 => esc_html__( 'No', 'cbxlatesttweets' ),
							),
							'default'  => 1
						),
						'enable_cache'  => array(
							'name'     => 'enable_cache',
							'label'    => esc_html__( 'Enable cache', 'cbxlatesttweets' ),
							'desc'     => esc_html__( 'Enable caching of tweets', 'cbxlatesttweets' ),
							'type'     => 'radio',
							'options'  => array(
								1 => esc_html__( 'Yes', 'cbxlatesttweets' ),
								0 => esc_html__( 'No', 'cbxlatesttweets' ),
							),
							'default'  => 1
						),
						'cache_time'       => array(
							'name'           => 'cache_time',
							'label'          => esc_html__( 'Default cache time (hour)', 'cbxlatesttweets' ),
							'desc'           => esc_html__( 'How many hour tweets wil be cached?', 'cbxlatesttweets' ),
							'type'           => 'number',
							'default'        => 2
						),

					),
					'cbxlatesttweets_tools'      => array(
						'delete_global_config' => array(
							'name'     => 'delete_global_config',
							'label'    => esc_html__( 'On Uninstall delete plugin data', 'cbxlatesttweets' ),
							'desc'     => '<p>' . __( 'Delete Global Config data created by this plugin on uninstall.', 'cbxlatesttweets' ) . ' ' . __( 'Details <a data-target="cbxlatesttweets_tool_fullreset" class="cbxlatesttweets_jump" href="#">option values</a> and <a data-target="cbxlatesttweets_tool_cachereset" class="cbxlatesttweets_jump" href="#">transient values</a> ', 'cbxlatesttweets' ) . '</p>' . '<p>' . __( '<strong>Please note that this process can not be undone and it is recommended to keep full database backup before doing this.</strong>', 'cbxlatesttweets' ) . '</p>',
							'type'     => 'radio',
							'options'  => array(
								'yes' => esc_html__( 'Yes', 'cbxlatesttweets' ),
								'no'  => esc_html__( 'No', 'cbxlatesttweets' ),
							),
							'default'  => 'no'
						),
						'reset_data'           => array(
							'name'     => 'reset_data',
							'label'    => esc_html__( 'Reset all data', 'cbxlatesttweets' ),
							'desc'     => sprintf( __( 'Reset option values created by this plugin. 
<a class="button button-primary" onclick="return confirm(\'%s\')" href="%s">Reset Data</a>', 'cbxlatesttweets' ), esc_html__( 'Are you sure to reset all data, this process can not be undone?', 'cbxlatesttweets' ), $reset_data_link ) . $table_data_html,
							'type'     => 'html',
							'default'  => 'off'
						),
						'reset_transients'           => array(
							'name'     => 'reset_transients',
							'label'    => esc_html__( 'Reset all cache data', 'cbxlatesttweets' ),
							'desc'     => sprintf( __( 'Reset cache data created created by this plugin. 
<a class="button button-primary" onclick="return confirm(\'%s\')" href="%s">Reset Cate</a>', 'cbxlatesttweets' ), esc_html__( 'Are you sure to reset all data, this process can not be undone?', 'cbxlatesttweets' ), $reset_cache_link ) . $table_cache_html,
							'type'     => 'html',
							'default'  => 'off'
						)


					)
				);

			$settings_fields = array(); //final setting array that will be passed to different filters

			$sections = $this->get_settings_sections();


			foreach ( $sections as $section )
			{
				if ( ! isset( $settings_builtin_fields[$section['id']] ) )
				{
					$settings_builtin_fields[$section['id']] = array();
				}
			}

			foreach ( $sections as $section )
			{
				$settings_builtin_fields_section_id = $settings_builtin_fields[$section['id']];
				$settings_fields[$section['id']]    = apply_filters( 'cbxlatesttweets_global_' . $section['id'] . '_fields', $settings_builtin_fields_section_id );
			}


			$settings_fields = apply_filters( 'cbxlatesttweets_global_fields', $settings_fields ); //final filter if need

			return $settings_fields;
		}

		/**
		 * Register the administration menu for this plugin into the WordPress Dashboard menu.
		 */
		public function plugin_admin_menu() {

			add_options_page( esc_html__( 'CBX Latest Tweets Configuration', 'cbxlatesttweets' ),
				esc_html__( 'CBX Latest Tweets', 'cbxlatesttweets' ), 'manage_options', 'cbxlatesttweetssettings', array(
					$this,
					'display_plugin_admin_page'
				) );
		}

		/**
		 * Render the settings page for this plugin.
		 *
		 * @since    1.0.0
		 */
		public function display_plugin_admin_page() {
			include( 'templates/admin-settings-display.php' );
		}

		/**
		 * Register the stylesheets for the admin area.
		 *
		 * @since    1.0.0
		 */
		public function enqueue_styles($hook) {

			if($hook == 'settings_page_cbxlatesttweetssettings'){
				wp_register_style( 'cbxlatesttweets-setting', plugin_dir_url( __FILE__ ) . '../assets/css/cbxlatesttweets-setting.css', array(), $this->version, 'all' );


				wp_enqueue_style( 'wp-color-picker' );
				wp_enqueue_style( 'cbxlatesttweets-setting' );

			}

		}//end method enqueue_styles

		/**
		 * Register the JavaScript for the admin area.
		 *
		 * @since    1.0.0
		 */
		public function enqueue_scripts($hook) {
			if($hook == 'settings_page_cbxlatesttweetssettings'){
				wp_register_script( 'cbxlatesttweets-setting', plugin_dir_url( __FILE__ ) . '../assets/js/cbxlatesttweets-setting.js', array( 'jquery' ), $this->version, true );

				wp_enqueue_media();
				wp_enqueue_script( 'wp-color-picker' );
				wp_enqueue_script( 'jquery' );

				wp_enqueue_script( 'cbxlatesttweets-setting' );

			}


		}//end method enqueue_scripts

		/**
		 * Full plugin reset and redirect
		 */
		public function plugin_fullreset(){

			$option_prefix = 'cbxlatesttweets_';

			CBXLatestTweetsHelper::deleteAllOptionNames();
			CBXLatestTweetsHelper::deleteAllTransientsNames();


			// create plugin's core table tables
			activate_cbxlatesttweets();



			//3rd party plugin's table creation
			do_action('cbxlatesttweets_plugin_reset', $option_prefix);


			$this->settings_api->set_sections($this->get_settings_sections());
			$this->settings_api->set_fields($this->get_settings_fields());
			$this->settings_api->admin_init();

			wp_safe_redirect(admin_url('options-general.php?page=cbxlatesttweetssettings#cbxlatesttweets_tools'));
			exit();
		}//end method plugin_fullreset

		/**
		 * Reset cache or transient caches
		 */
		public function plugin_cachereset(){
			CBXLatestTweetsHelper::deleteAllTransientsNames();

			wp_safe_redirect(admin_url('options-general.php?page=cbxlatesttweetssettings#cbxlatesttweets_tools'));
			exit();
		}//end method plugin_cachereset

		/**
		 * Add settings action link to the plugins page.
		 *
		 * @since    1.0.0
		 */
		public function plugin_listing_setting_link($links)
		{

			return array_merge(
				array(
					'settings' => '<a target="_blank" href="' . admin_url('options-general.php?page=cbxlatesttweetssettings') . '">' . esc_html__('Settings', 'cbxlatesttweets') . '</a>'
				),
				$links
			);

		}//end method plugin_listing_setting_link

	}//end method CBXLatestTweets_Admin
