<?php

	/**
	 * The public-facing functionality of the plugin.
	 *
	 * @link       https://www.codeboxr.com
	 * @since      1.0.0
	 *
	 * @package    CBXLatestTweets
	 * @subpackage CBXLatestTweets/public
	 */

	/**
	 * The public-facing functionality of the plugin.
	 *
	 * Defines the plugin name, version, and two examples hooks for how to
	 * enqueue the public-facing stylesheet and JavaScript.
	 *
	 * @package    CBXLatestTweets
	 * @subpackage CBXLatestTweets/public
	 * @author     Codeboxr <info@codeboxr.com>
	 */
	class CBXLatestTweets_Public {

		/**
		 * The ID of this plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 * @var      string $plugin_name The ID of this plugin.
		 */
		private $plugin_name;

		/**
		 * The version of this plugin.
		 *
		 * @since    1.0.0
		 * @access   private
		 * @var      string $version The current version of this plugin.
		 */
		private $version;

		/**
		 * Initialize the class and set its properties.
		 *
		 * @since    1.0.0
		 *
		 * @param      string $plugin_name The name of the plugin.
		 * @param      string $version     The version of this plugin.
		 */
		public function __construct( $plugin_name, $version ) {

			$this->plugin_name = $plugin_name;
			$this->version     = $version;

			//add shortcode for frontend latest tweets lists, same latest tweets list is available as widget
			add_shortcode( 'cbxlatesttweets', array(
				$this,
				'cbxlatesttweets_shortcode'
			) );

			//get instance of setting api
			$this->settings_api = new CBXLatestTweets_Settings_API( $this->plugin_name, $this->version );


		}

		/**
		 * Shortcode callback for latest tweets
		 */
		public function cbxlatesttweets_shortcode( $atts ) {

			$include_rts      = $this->settings_api->get_option( 'include_rts', 'cbxlatesttweets_config', 0 );
			$exclude_replies  = $this->settings_api->get_option( 'exclude_replies', 'cbxlatesttweets_config', 1 );
			$time_format      = $this->settings_api->get_option( 'time_format', 'cbxlatesttweets_config', 1 );
			$date_time_format = $this->settings_api->get_option( 'date_time_format', 'cbxlatesttweets_config', CBXLatestTweetsHelper::getGlobalDateTimeFormat() );


			$atts = shortcode_atts(
				array(
					'scope'            => 'shortcode',
					'limit'            => 10,
					'username'         => '',
					'include_rts'      => $include_rts,
					'exclude_replies'  => $exclude_replies,
					'time_format'      => $time_format,
					'date_time_format' => $date_time_format
				), $atts, 'cbxlatesttweets' );


			$scope           = ( isset( $atts['scope'] ) && $atts['scope'] != '' ) ? esc_attr( $atts['scope'] ) : 'shortcode';
			$limit           = intval( $atts['limit'] );
			$username        = esc_attr( $atts['username'] );
			$include_rts     = intval( $atts['include_rts'] );
			$exclude_replies = intval( $atts['exclude_replies'] );

			$time_format      = intval( $atts['time_format'] );
			$date_time_format = $atts['date_time_format'];

			$limit = ( intval( $limit ) > 20 ) ? 20 : $limit; //maximum 20 tweets per page

			$latest_tweets = CBXLatestTweetsHelper::getLatestTweets( $username, $limit, $include_rts, $exclude_replies );

			CBXLatestTweetsHelper::enqueue_styles();

			// Display the most rated post link
			ob_start();
			//include( plugin_dir_path( __FILE__ ) . '../widgets/cbxlatesttweets-widget/views/public.php' );
			include (CBXLatestTweetsHelper::find_template());
			$content = ob_get_contents();
			ob_end_clean();

			return $content;

		}


		/**
		 * Register Widget
		 */
		public function register_widget() {
			register_widget( "CBXLatestTweetsWidget" ); //form widget
		}

		/**
		 * Register the stylesheets for the public-facing side of the site.
		 *
		 * @since    1.0.0
		 */
		public function enqueue_styles() {

			wp_register_style( 'cbxlatesttweets-public', plugin_dir_url( __FILE__ ) . '../assets/css/cbxlatesttweets-public.css', array(), $this->version, 'all' );
			//wp_enqueue_style( 'cbxlatesttweets-public');

		}

		/**
		 * Register the JavaScript for the public-facing side of the site.
		 *
		 * @since    1.0.0
		 */
		/*public function enqueue_scripts() {

			wp_register_script( 'cbxlatesttweets-public', plugin_dir_url( __FILE__ ) . 'js/cbxlatesttweets-public.js', array( 'jquery' ), $this->version, false );
			wp_enqueue_script( 'cbxlatesttweets-public');

		}*/

	}
